package com.unissoft.controller;

import com.unissoft.MyConstant;
import com.unissoft.config.MinioConfig;
import com.unissoft.model.SysFileMetadata;
import com.unissoft.result.ResultView;
import com.unissoft.service.FileUploadService;
import com.unissoft.service.MinioService;
import com.unissoft.service.SysFileMetadataService;
import io.minio.ObjectStat;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.tomcat.util.http.fileupload.IOUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.InputStream;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;

@RequestMapping("/file")
@RestController
@Slf4j
public class FileController {

    @Autowired
    private MinioService minioService;

    @Autowired
    private MinioConfig minioConfig;

    @Autowired
    private FileUploadService fileUploadService;

    @Autowired
    private SysFileMetadataService fileMetadataService;

    @ApiOperation(value = "单文件上传", notes = "")
    @PostMapping("/uploadFile")
    public ResultView uploadFile(MultipartFile file) {
        String filepath = fileUploadService.uploadFile(file);
        return ResultView.success(filepath);
    }

    @ApiOperation(value = "多文件上传", notes = "")
    @PostMapping("/uploadFiles")
    public ResultView uploadFiles(@RequestParam("multipartFiles") List<MultipartFile> files) {
        if(null != files && files.isEmpty() == false) {
            List<String> list = new ArrayList<>();
            for (MultipartFile file : files) {
                String filepath = fileUploadService.uploadFile(file);
                list.add(filepath);
            }
            return ResultView.success(list);
        }
        return ResultView.error("上传文件为空");
    }

    @ApiOperation(value = "文件下载", notes = "")
    @GetMapping("/download")
    public ResultView download(HttpServletResponse response, @RequestParam("filepath") String filepath) {
        try {
            String bucketName = MyConstant.ATTACHMENT_PATH;
            //获取对象信息和对象的元数据。
            ObjectStat objectStat = minioService.statObject(bucketName, filepath);
            //setContentType 设置发送到客户机的响应的内容类型
            response.setContentType(objectStat.contentType());
            //设置响应头
            String filename = objectStat.name();
            SysFileMetadata fileMetadata = fileMetadataService.getById(filepath);
            if(null != fileMetadata) {
                filename = fileMetadata.getFilename();
            }
            response.setHeader("Content-Disposition", "attachment;filename=" + URLEncoder.encode(filename, "UTF-8"));
            //文件流
            InputStream object = minioService.getObject(bucketName, filepath);
            //设置文件大小
            response.setHeader("Content-Length", String.valueOf(objectStat.length()));
            IOUtils.copy(object, response.getOutputStream());
            //关闭流
            object.close();
            return ResultView.success();
        } catch (Exception e) {
            log.error("下载文件失败,错误信息: " + e.getMessage());
            return ResultView.error("下载文件失败,错误信息: " + e.getMessage());
        }
    }

    @ApiOperation(value = "文件删除", notes = "")
    @DeleteMapping(value = "/deleteFile")
    public ResultView deleteFile(@RequestParam("objectName") String objectName) {
        String bucketName = MyConstant.ATTACHMENT_PATH;
        minioService.removeObject(bucketName, objectName);
        return ResultView.success();
    }

    @ApiOperation(value = "预览文件", notes = "")
    @GetMapping("/preview")
    public ResultView getPreviewFile(@RequestParam("expires") Integer expires,
                                     @RequestParam("filepath") String filepath) {
        //获取存储捅名
        String bucketName = MyConstant.ATTACHMENT_PATH;
        //生成一个给HTTP GET请求用的presigned URL
        String urlPath = minioService.presignedGetObject(bucketName, filepath, expires);
        //封装信息
        return ResultView.success(urlPath);
    }



}