package com.unissoft.controller;


import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.unissoft.model.SysAnnouncement;
import com.unissoft.model.SysUserPO;
import com.unissoft.result.PageParam;
import com.unissoft.result.ResultView;
import com.unissoft.service.SysAnnouncementService;
import com.unissoft.utils.UserUtils;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * <p>
 * 公告表 前端控制器
 * </p>
 *
 * @author root
 * @since 2020-12-10
 */
@RestController
@RequestMapping("/sysAnnouncement")
public class SysAnnouncementController {

    @Autowired
    private SysAnnouncementService announcementService;


    @ApiOperation(value = "已发布公告分页条件查询", notes = "提交参数：{\n" +
            "  \"condition\":\"{'publishTimeStart':'','publishTimeEnd':'','title':'', 'annoType':''}\",\n" +
            "  \"pageIndex\": 1,\n" +
            "  \"pageSize\": 10\n" +
            "}")
    @PostMapping("/authority/getPublishedPage")
    public ResultView getPublishedPage(@RequestBody PageParam pageParam) {
        SysUserPO userPO = UserUtils.getCurrUserInfo();
        Page<SysAnnouncement> page = announcementService.getPage(pageParam, true, userPO.getuId());
        return ResultView.success(page);
    }

    @ApiOperation(value = "草稿箱公告分页条件查询", notes = "提交参数：{\n" +
            "  \"condition\":\"{'title':''}\",\n" +
            "  \"pageIndex\": 1,\n" +
            "  \"pageSize\": 10\n" +
            "}")
    @PostMapping("/authority/getDraftPage")
    public ResultView getDraftPage(@RequestBody PageParam pageParam) {
        SysUserPO userPO = UserUtils.getCurrUserInfo();
        Page<SysAnnouncement> page = announcementService.getPage(pageParam, false, userPO.getuId());
        return ResultView.success(page);
    }

    @ApiOperation(value = "根据id查询", notes = "")
    @GetMapping("/authority/getById/{id}")
    public ResultView getById(@PathVariable int id) {
        SysAnnouncement model = announcementService.getById(id);
        return ResultView.success(model);
    }

    @ApiOperation(value = "新增", notes = "")
    @PostMapping("/authority_button/add")
    public ResultView add(@RequestBody SysAnnouncement model) {
        SysUserPO userPO = UserUtils.getCurrUserInfo();
        announcementService.add(model, userPO);
        return ResultView.success(model);
    }

    @ApiOperation(value = "修改", notes = "")
    @PostMapping("/authority_button/update")
    public ResultView update(@RequestBody SysAnnouncement model) {
        SysUserPO userPO = UserUtils.getCurrUserInfo();
        announcementService.updateById(model, userPO);
        return ResultView.success();
    }

    @ApiOperation(value = "根据id删除", notes = "")
    @DeleteMapping("/authority_button/deleteById")
    public ResultView deleteById(@RequestParam int id) {
        announcementService.removeAnnouncementById(id);
        return ResultView.success();
    }

}
