package com.unissoft.controller;


import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.unissoft.dto.SendNoticeVO;
import com.unissoft.model.SysNotice;
import com.unissoft.model.SysNoticeUser;
import com.unissoft.model.SysUserPO;
import com.unissoft.result.PageParam;
import com.unissoft.result.ResultView;
import com.unissoft.service.SysNoticeUserService;
import com.unissoft.utils.UserUtils;
import com.unissoft.vo.SysNoticeUserVO;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;

/**
 * <p>
 *  前端控制器
 * </p>
 *
 * @author root
 * @since 2020-12-07
 */
@RestController
@RequestMapping("/sysNoticeUser")
public class SysNoticeUserController {

    @Autowired
    private SysNoticeUserService noticeUserService;

    @ApiOperation(value = "分页条件查询", notes = "提交参数：{\n" +
            "  \"condition\":\"{'readStatus':'','noticeType':'','content':''}\",\n" +
            "  \"pageIndex\": 1,\n" +
            "  \"pageSize\": 10\n" +
            "}")
    @PostMapping("/authority/getPage")
    public ResultView getPage(@RequestBody PageParam pageParam) {
        SysUserPO userPO = UserUtils.getCurrUserInfo();
        Page<SysNoticeUserVO> page = noticeUserService.getPage(pageParam, userPO.getuId());
        return ResultView.success(page);
    }

    @ApiOperation(value = "根据id查询", notes = "")
    @GetMapping("/authority/getById/{id}")
    public ResultView getById(@PathVariable int id) {
        SysNoticeUser model = noticeUserService.getById(id);
        return ResultView.success(model);
    }

    @ApiOperation(value = "新增", notes = "")
    @PostMapping("/authority_button/add")
    public ResultView add(@RequestBody SysNoticeUser model) {
        SysUserPO userPO = UserUtils.getCurrUserInfo();
        model.setCreateBy(userPO.getuId());
        model.setUpdateBy(userPO.getuId());

        Date date = new Date();
        model.setCreateTime(date);
        model.setUpdateTime(date);

        noticeUserService.saveNoticeUser(model);
        return ResultView.success(model);
    }

    @ApiOperation(value = "修改", notes = "")
    @PostMapping("/authority_button/update")
    public ResultView update(@RequestBody SysNoticeUser model) {
        SysUserPO userPO = UserUtils.getCurrUserInfo();
        model.setUpdateBy(userPO.getuId());

        Date date = new Date();
        model.setUpdateTime(date);

        noticeUserService.updateNoticeUser(model);
        return ResultView.success();
    }

    @ApiOperation(value = "根据id删除", notes = "")
    @DeleteMapping("/authority_button/deleteById")
    public ResultView deleteById(@RequestParam int id) {
        noticeUserService.removeById(id);
        return ResultView.success();
    }

    @ApiOperation(value = "发送通知", notes = "")
    @PostMapping("/authority_button/sendNotice")
    public ResultView sendNotice(@RequestBody SendNoticeVO sendNoticeVO) {
        noticeUserService.sendNotice(sendNoticeVO);
        return ResultView.success();
    }

    @ApiOperation(value = "标记已读状态", notes = "")
    @GetMapping("/authority_button/readed/{id}")
    public ResultView readed(@PathVariable int id) {
        noticeUserService.setReadedById(id);
        return ResultView.success();
    }

    @ApiOperation(value = "查看通知详情", notes = "")
    @GetMapping("/authority/viewNoticeById/{id}")
    public ResultView viewNoticeById(@PathVariable int id) {
        SysNotice sysNotice = noticeUserService.viewNoticeById(id);
        return ResultView.success(sysNotice);
    }

}
