package com.unissoft.feign;

import com.unissoft.MyConstant;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.Map;

/**
 * feign调用SystemManage服务
 */
@FeignClient(value = MyConstant.CSC_SZLS_UPMS_SYSTEM_MANAGE, fallback = SystemManageHystrix.class)
@Component
public interface SystemManageFeign {

    /**
     * 根据depotId、recvType、recvDetails 获取相关用户信息
     * @param depotId   分公司或粮库的id
     * @param recvType  值定义： 10：全员，20：指定粮库，30：分公司全员，40：指定角色组
     * @param recvDetails   如果recvType=20，则存粮库的组织id，多个组织id用逗号分隔，如果recvType=40，则存角色组的角色id，多个角色id用逗号分隔
     * @return
     */
    @GetMapping("/user/getNoticeUserInfo")
    Map<Integer, String> getNoticeUserInfo(@RequestParam Integer depotId, @RequestParam String recvType, @RequestParam(required = false) String recvDetails);

    /**
     * 根据角色IDs，获取用户信息（用户id，用户姓名）集合，
     * @param depotId   库id，不为空时，查具体库下角色对应的用户
     * @param roleIds   角色id，多个id用逗号隔开
     * @return
     */
    @GetMapping("/user/getUsersByRoleIds")
    Map<Integer, String> getUsersByRoleIds(@RequestParam(required = false) Integer depotId, @RequestParam String roleIds);

}
