package com.unissoft.job;

import com.alibaba.fastjson.JSONObject;
import com.unissoft.constant.GrainConstant;
import com.unissoft.dto.SendNoticeVO;
import com.unissoft.dto.SysNoticeVO;
import com.unissoft.dto.ToBeDoneCallbackVO;
import com.unissoft.enums.NoticeTypeEnum;
import com.unissoft.feign.BusinessFeign;
import com.unissoft.service.QuartzService;
import com.unissoft.service.SysNoticeUserService;
import org.apache.commons.lang.StringUtils;
import org.quartz.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.io.Serializable;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@DisallowConcurrentExecution
@Component
public class ToBeDoneNoticeWarnJob implements Job, Serializable {

    private static final long serialVersionUID = 1L;

    @Autowired
    private BusinessFeign businessFeign;

    @Autowired
    private SysNoticeUserService sysNoticeUserService;

    @Autowired
    private QuartzService quartzService;

    @Override
    public void execute(JobExecutionContext context) {
        JobDetail jobDetail = context.getJobDetail();
        JobDataMap dataMap = jobDetail.getJobDataMap();

        List<ToBeDoneCallbackVO> warnNotices = JSONObject.parseArray(dataMap.getString("warnNotices"), ToBeDoneCallbackVO.class);
        String title = dataMap.getString("title");
        String jobName = dataMap.getString(GrainConstant.STR_JOBNAME);
        String group = dataMap.getString("group");
        String triggerName = dataMap.getString("triggerName");

        Map<Integer, ToBeDoneCallbackVO> map = new HashMap<>();
        for(ToBeDoneCallbackVO vo : warnNotices) {
            map.put(vo.getToBeDoneId(), vo);
        }
        //
        String toBeDoneIds = StringUtils.join(map.keySet(), ",");
        List<Integer> list = businessFeign.getNotBeDoneNotice(toBeDoneIds);

        //生成预警信息
        if(null != list && list.isEmpty() == false) {
            for(Integer id : list) {
                ToBeDoneCallbackVO warnNotice = map.get(id);

                SendNoticeVO sendNoticeVO = new SendNoticeVO();
                Map<Integer, String> users = new HashMap<>();
                users.put(warnNotice.getUserId(), warnNotice.getPersonnelName());
                sendNoticeVO.setUsers(users);

                SysNoticeVO sysNoticeVO = new SysNoticeVO();
                sysNoticeVO.setNoticeType(NoticeTypeEnum.WARN.getCode());
                sysNoticeVO.setNoticeTime(new Date());
                sysNoticeVO.setTitle(title);


                StringBuilder stringBuilder = new StringBuilder();
                stringBuilder.append(warnNotice.getDepotName()).append("-");
                if(StringUtils.isNotBlank(warnNotice.getHouseName())) {
                    stringBuilder.append(warnNotice.getHouseName()).append("-");
                }
                stringBuilder.append("即将逾期");

                String content = stringBuilder.toString();
                sysNoticeVO.setContent(content);
                sysNoticeVO.setWarehouse(warnNotice.getHouseName());
                sysNoticeVO.setOrgName(warnNotice.getDepotName());
                sendNoticeVO.setSysNoticeVO(sysNoticeVO);
                sysNoticeUserService.sendNotice(sendNoticeVO);
            }
        }

        //删除任务调度
        quartzService.removeJob(jobName, triggerName, group);
    }

}
