package com.unissoft.redismq;

import com.unissoft.controller.WebSocketServer;
import com.unissoft.dto.MessageVO;
import com.unissoft.service.SysMessageService;
import com.unissoft.utils.JsonUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.connection.Message;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.listener.adapter.MessageListenerAdapter;
import org.springframework.stereotype.Component;

/**
 * mqMsgChannel 订阅频道处理类
 */
@Slf4j
@Component
public class MsgListenerHandle extends MessageListenerAdapter {

    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @Autowired
    private WebSocketServer webSocketServer;

    @Autowired
    private SysMessageService sysMessageService;

    /**
     * 收到监听消息
     * @param message
     * @param bytes
     */
    @Override
    public void onMessage(Message message, byte[] bytes) {
        byte[] body = message.getBody();
        byte[] channel = message.getChannel();
        String messageStr;
        String topic;
        try {
            messageStr = redisTemplate.getStringSerializer().deserialize(body);
            topic = redisTemplate.getStringSerializer().deserialize(channel);
            log.info("收到主题:" + topic + ", 信息内容：" + messageStr);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return;
        }

        MessageVO messageVO = JsonUtil.parseJsonToObj(messageStr, MessageVO.class);
        if(WebSocketServer.containsUser(messageVO.getToUserId())) {
            boolean pushedStatus = webSocketServer.sendUserMessage(messageVO.getToUserId(), messageStr);
            if(pushedStatus) {
                sysMessageService.setPushedStatus(messageVO.getId());
            }
        }
    }
}
