package com.unissoft.redismq;

import com.unissoft.constant.GrainConstant;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.redis.connection.RedisConnectionFactory;
import org.springframework.data.redis.listener.PatternTopic;
import org.springframework.data.redis.listener.RedisMessageListenerContainer;

@Configuration
public class RedisListenerBean {

    /**
     * redis消息监听器容器
     * 可以添加多个监听不同话题的redis监听器，只需要把消息监听器和相应的消息订阅处理器绑定，该消息监听器
     * 通过反射技术调用消息订阅处理器的相关方法进行一些业务处理
     * @param connectionFactory
     * @param msgListenerHandle
     * @param userStatusListenerHandle
     * @return
     */
    @Bean
    public RedisMessageListenerContainer container(RedisConnectionFactory connectionFactory,
                                                   MsgListenerHandle msgListenerHandle,
                                                   UserStatusListenerHandle userStatusListenerHandle){
        RedisMessageListenerContainer container = new RedisMessageListenerContainer();
        container.setConnectionFactory(connectionFactory);
        //订阅频道，可订阅多个
        container.addMessageListener(msgListenerHandle, new PatternTopic(GrainConstant.MQ_MSG_CHANNEL));
        container.addMessageListener(userStatusListenerHandle, new PatternTopic(GrainConstant.MQ_USER_STATUS_CHANNEL));
        return container;
    }

}
