package com.unissoft.redismq;

import com.unissoft.constant.GrainConstant;
import com.unissoft.controller.WebSocketServer;
import com.unissoft.enums.OnlineStatusEnum;
import com.unissoft.utils.JsonUtil;
import com.unissoft.vo.UserOnlineStatusVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.connection.Message;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.listener.adapter.MessageListenerAdapter;
import org.springframework.stereotype.Component;

/**
 * mqUserStatusChannel 订阅频道处理类
 */
@Slf4j
@Component
public class UserStatusListenerHandle extends MessageListenerAdapter {

    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @Autowired
    private WebSocketServer webSocketServer;

    /**
     * 收到监听消息
     * @param message
     * @param bytes
     */
    @Override
    public void onMessage(Message message, byte[] bytes) {
        byte[] body = message.getBody();
        byte[] channel = message.getChannel();
        String rawMsg;
        String topic;
        try {
            rawMsg = redisTemplate.getStringSerializer().deserialize(body);
            topic = redisTemplate.getStringSerializer().deserialize(channel);
            log.info("收到主题:" + topic + ", 信息内容：" + rawMsg);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            return;
        }
        //
        UserOnlineStatusVO onlineStatusVO = JsonUtil.parseJsonToObj(rawMsg, UserOnlineStatusVO.class);
        if(onlineStatusVO.getOnlineStatus() == OnlineStatusEnum.OFFLINE.getCode()) {
            if(WebSocketServer.containsUser(onlineStatusVO.getUserId())) {
                WebSocketServer.removeSession(onlineStatusVO.getUserId());
            }
        } else if(onlineStatusVO.getOnlineStatus() == OnlineStatusEnum.STATISTICS.getCode()) {//统计在线人数
            long onlineNum = redisTemplate.opsForSet().size(GrainConstant.ONLINE_USERS);//在线人数
            //通知所有用户 在线人数
            webSocketServer.noticeOnlineNum2All(onlineNum);
        }

    }
}
