package com.unissoft.service.impl;

import cn.hutool.core.io.file.FileNameUtil;
import com.unissoft.MyConstant;
import com.unissoft.config.MinioConfig;
import com.unissoft.model.SysFileMetadata;
import com.unissoft.service.FileUploadService;
import com.unissoft.service.MinioService;
import com.unissoft.service.SysFileMetadataService;
import com.unissoft.utils.DateUtil;
import lombok.SneakyThrows;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.text.DecimalFormat;
import java.util.Date;
import java.util.UUID;

@Service
public class FileUploadServiceImpl implements FileUploadService {

    @Autowired
    private MinioService minioService;

    @Autowired
    private MinioConfig minioConfig;

    @Autowired
    private SysFileMetadataService fileMetadataService;

    /**
     * 上传文件至minio
     * @param file  要上传的文件
     * @return  返回保存在minio的文件名
     */
    @Override
    public String uploadFile(MultipartFile file) {
        String originalFilename = file.getOriginalFilename();       // 原始文件名
        String extName = FileNameUtil.extName(originalFilename);    // 文件扩展名
        String minioFilename = generateMinioFilename(extName);      // 生成要保存在minio的文件名

        //上传文件
        uploadFile2Minio(file, minioFilename);

        //保存至表
        SysFileMetadata fileMetadata = new SysFileMetadata();
        fileMetadata.setFilepath(minioFilename);
        fileMetadata.setFilename(originalFilename);//文件名称
        fileMetadata.setFilesize(formatFileSize(file.getSize()));//文件大小
        fileMetadata.setCreateTime(new Date());
        fileMetadataService.save(fileMetadata);
        //
        return minioFilename;
    }

    /**
     * 生成要保存在minio的文件名
     * @param extName 文件扩展名
     * @return
     */
    private String generateMinioFilename(String extName) {
        // 定义文件路径
        String format = DateUtil.formatDateByPattern(new Date(), "yyyy/MM/dd/");
        String uuid = UUID.randomUUID().toString().replaceAll("-", "");
        return format + uuid + "." + extName;
    }

    @SneakyThrows
    private void uploadFile2Minio(MultipartFile file, String minioFilename) {
        String bucketName = MyConstant.ATTACHMENT_PATH; //默认存储桶
        // 判断是否存在该存储桶
        if (! minioService.bucketExists(bucketName)) {
            // 不存在则创建
            minioService.makeBucket(bucketName);
            // 设置存储桶只读策略
            String version = DateUtil.formatDateByPattern(new Date(), DateUtil.PATTERN_DATE);
            String bucketPolicy ="{\"Version\":\"" + version + "\",\"Statement\":[{\"Effect\":\"Allow\",\"Principal\":{\"AWS\":[\"*\"]},\"" +
                    "Action\":[\"s3:GetBucketLocation\",\"s3:ListBucket\"],\"Resource\":[\"arn:aws:s3:::fast\"]},{\"Effect\":\"Allow\",\"" +
                    "Principal\":{\"AWS\":[\"*\"]},\"Action\":[\"s3:GetObject\"],\"Resource\":[\"arn:aws:s3:::" +
                    bucketName+"/*\"]}]}";
            // 设置存储桶策略
            minioService.setBucketPolicy(bucketName, bucketPolicy);
        }
        //上传文件
        minioService.putObject(bucketName, file, minioFilename);
    }

    /**
     * 显示文件大小信息单位
     *
     * @param fileS
     * @return
     */
    private static String formatFileSize(long fileS) {
        DecimalFormat df = new DecimalFormat("#.00");
        String fileSizeString = "";
        String wrongSize = "0B";
        if (fileS == 0) {
            return wrongSize;
        }
        if (fileS < 1024) {
            fileSizeString = df.format((double) fileS) + " B";
        } else if (fileS < 1048576) {
            fileSizeString = df.format((double) fileS / 1024) + " KB";
        } else if (fileS < 1073741824) {
            fileSizeString = df.format((double) fileS / 1048576) + " MB";
        } else {
            fileSizeString = df.format((double) fileS / 1073741824) + " GB";
        }
        return fileSizeString;
    }

}
