package com.unissoft.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.unissoft.MyConstant;
import com.unissoft.dto.MessageVO;
import com.unissoft.enums.MessageTypeEnum;
import com.unissoft.feign.SystemManageFeign;
import com.unissoft.mapper.SysAnnouncementMapper;
import com.unissoft.model.SysAnnouncement;
import com.unissoft.model.SysAnnouncementUser;
import com.unissoft.model.SysUserPO;
import com.unissoft.result.PageParam;
import com.unissoft.service.SysAnnouncementService;
import com.unissoft.service.SysAnnouncementUserService;
import com.unissoft.service.SysMessageService;
import com.unissoft.utils.DateUtil;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.time.DateUtils;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 公告表 服务实现类
 * </p>
 *
 * @author root
 * @since 2020-12-10
 */
@Service
public class SysAnnouncementServiceImpl extends ServiceImpl<SysAnnouncementMapper, SysAnnouncement> implements SysAnnouncementService {

    @Autowired
    private SysAnnouncementMapper announcementMapper;

    @Autowired
    private SysAnnouncementUserService announcementUserService;

    @Autowired
    private SystemManageFeign systemManageFeign;

    @Autowired
    private SysMessageService sysMessageService;


    @Override
    public Page<SysAnnouncement> getPage(PageParam pageParam, boolean publishStatus, Integer userId) {
        Integer pageIndex = pageParam.getPageIndex();
        Integer pageSize = pageParam.getPageSize();
        QueryWrapper<SysAnnouncement> ew = new QueryWrapper<>();
        ew.eq("publish_status", publishStatus ? 1 : 0);//发布状态
        if(null != userId) {
            ew.eq("publish_user_id", userId);
        }

        if(! StringUtils.isEmpty(pageParam.getCondition())){
            JSONObject object = new JSONObject(pageParam.getCondition());

            if (object.has("publishTimeStart")) {
                String publishTimeStartStr = object.getString("publishTimeStart");
                if (! StringUtils.isEmpty(publishTimeStartStr)) {
                    Date publishTimeStart = DateUtil.formatDate(publishTimeStartStr, "yyyy-MM-dd");
                    ew.ge("publish_time", publishTimeStart);
                }
            }

            if (object.has("publishTimeEnd")) {
                String publishTimeEndStr = object.getString("publishTimeEnd");
                if (! StringUtils.isEmpty(publishTimeEndStr)) {
                    Date publishTimeEnd = DateUtil.formatDate(publishTimeEndStr, "yyyy-MM-dd");
                    publishTimeEnd = DateUtils.addDays(publishTimeEnd, 1);//加1天
                    ew.lt("publish_time", publishTimeEnd);
                }
            }

            if (object.has("title")) {
                String title = object.getString("title");
                if (! StringUtils.isEmpty(title)) {
                    ew.like("title", title.trim());
                }
            }

            if (object.has("annoType")) {
                String annoType = object.getString("annoType");
                if (! StringUtils.isEmpty(annoType)) {
                    ew.eq("anno_type", annoType.trim());
                }
            }
        }
        ew.orderByDesc("publish_time");
        Page page = new Page();
        if (pageIndex != MyConstant.ZERO && pageSize != MyConstant.ZERO) {
            page = new Page(pageIndex, pageSize);
        }
        Page<SysAnnouncement> pageResult = announcementMapper.selectPage(page, ew);
        return pageResult;

    }

    @Override
    @Transactional
    public boolean add(SysAnnouncement model, SysUserPO userPO) {
        model.setPublishUserId(userPO.getuId());
        model.setPublishUserName(userPO.getPersonnelName());
        model.setPublishOrgId(userPO.getDepotId());
        model.setPublishOrgName(userPO.getDepotName());

        model.setCreateBy(userPO.getuId());
        model.setUpdateBy(userPO.getuId());

        Date date = new Date();
        model.setCreateTime(date);
        model.setUpdateTime(date);

        Boolean publishStatus = model.getPublishStatus();
        if(publishStatus) {
            model.setPublishTime(date);//发布时间为新增时的系统时间
        }
        save(model);

        if(publishStatus) {
            saveAnnouncementUsersAndSendMessage(model, userPO);
        }

        return true;
    }

    @Override
    @Transactional
    public boolean updateById(SysAnnouncement model, SysUserPO userPO) {
        model.setPublishUserId(userPO.getuId());
        model.setPublishUserName(userPO.getPersonnelName());
        model.setPublishOrgId(userPO.getDepotId());
        model.setPublishOrgName(userPO.getDepotName());

        model.setUpdateBy(userPO.getuId());

        Date date = new Date();
        model.setUpdateTime(date);

        Boolean publishStatus = model.getPublishStatus();
        if(publishStatus) {
            model.setPublishTime(date);//发布时间为新增时的系统时间
        }
        updateById(model);

        if(publishStatus) {
            saveAnnouncementUsersAndSendMessage(model, userPO);
        }

        return true;
    }

    private void saveAnnouncementUsers(int annoId, Map<Integer, String> users) {

        if(null != users && users.isEmpty() == false) {
            List<SysAnnouncementUser> list = new ArrayList<>();
            for(Integer userId : users.keySet()) {
                SysAnnouncementUser announcementUser = new SysAnnouncementUser();
                announcementUser.setAnnoId(annoId);
                announcementUser.setUserId(userId);
                announcementUser.setUserName(users.get(userId));
                announcementUser.setReadStatus(false);//设置未读

                Date date = new Date();
                announcementUser.setCreateTime(date);
                announcementUser.setUpdateTime(date);

                list.add(announcementUser);
            }
            announcementUserService.batchSave(list);
        }

    }

    private Map<Integer, String> getNoticeUserInfo(Integer depotId, String recvType, String recvDetails) {
        Map<Integer, String> users = systemManageFeign.getNoticeUserInfo(depotId, recvType, recvDetails);
        return users;
    }

    /**
     *
     * @param model
     * @param userPO
     */
    private void saveAnnouncementUsersAndSendMessage(SysAnnouncement model, SysUserPO userPO) {
        Map<Integer, String> users = getNoticeUserInfo(userPO.getDepotId(), model.getRecvType(), model.getRecvDetails());
        if(null != users && users.isEmpty() == false) {
            saveAnnouncementUsers(model.getId(), users);

            MessageVO messageVO = new MessageVO();
            messageVO.setTitle(model.getTitle());
            messageVO.setContent(model.getTitle());
            messageVO.setType(MessageTypeEnum.ATTENTION.getCode());
            messageVO.setFromUserId(userPO.getuId());
            sysMessageService.batchSendMessage(messageVO, users.keySet());
        }
    }

    @Override
    @Transactional
    public void removeAnnouncementById(int id) {
        removeById(id);
        //删除公告用户表
        announcementUserService.removeByAnnouncementId(id);
    }

}
