package com.unissoft.service.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Maps;
import com.unissoft.MyConstant;
import com.unissoft.constant.GrainConstant;
import com.unissoft.mapper.SysAnnouncementUserMapper;
import com.unissoft.model.SysAnnouncement;
import com.unissoft.model.SysAnnouncementUser;
import com.unissoft.result.PageParam;
import com.unissoft.service.SysAnnouncementService;
import com.unissoft.service.SysAnnouncementUserService;
import com.unissoft.vo.SysAnnouncementUserVO;
import org.apache.commons.lang.StringUtils;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 公告用户表 服务实现类
 * </p>
 *
 * @author root
 * @since 2020-12-10
 */
@Service
public class SysAnnouncementUserServiceImpl extends ServiceImpl<SysAnnouncementUserMapper, SysAnnouncementUser> implements SysAnnouncementUserService {

    @Autowired
    private SysAnnouncementUserMapper announcementUserMapper;

    @Autowired
    private SysAnnouncementService announcementService;

    @Override
    public Page<SysAnnouncementUserVO> getPage(PageParam pageParam, Integer userId) {
        Integer pageIndex = pageParam.getPageIndex();
        Integer pageSize = pageParam.getPageSize();
        Map<String, Object> map = Maps.newHashMap();
        if (null != userId) {
            map.put("userId", userId);
        }

        if (! StringUtils.isEmpty(pageParam.getCondition())) {
            JSONObject object = new JSONObject(pageParam.getCondition());

            if (object.has("readStatus")) {
                Boolean readStatus = object.getBoolean("readStatus");
                if (null != readStatus) {
                    map.put("readStatus", readStatus ? 1 : 0 );
                }
            }

            if (object.has("title")) {
                String title = object.getString("title");
                if (! StringUtils.isEmpty(title)) {
                    map.put("title", "%"+title.trim()+"%");
                }
            }

            if (object.has(GrainConstant.STR_CONTENT)) {
                String content = object.getString(GrainConstant.STR_CONTENT);
                if (! StringUtils.isEmpty(content)) {
                    map.put(GrainConstant.STR_CONTENT, "%"+content.trim()+"%");
                }
            }
        }

        Page page = new Page();
        if (pageIndex != MyConstant.ZERO && pageSize != MyConstant.ZERO) {
            page = new Page(pageIndex, pageSize);
        }
        Page<SysAnnouncementUserVO> pageResult = announcementUserMapper.getPage(page, map);

        return pageResult;
    }

    @Override
    @Transactional
    public int batchSave(List<SysAnnouncementUser> list) {
        return baseMapper.insertBatch(list);
    }

    @Override
    public void removeByAnnouncementId(int annoId) {
        Map<String, Object> columnMap = new HashMap<>();
        columnMap.put("anno_id", annoId);
        removeByMap(columnMap);
    }

    @Override
    public SysAnnouncement viewAnnouncementById(int id) {
        SysAnnouncementUser announcementUser = getById(id);
        //设置已读
        setReaded(announcementUser);
        //获取公告详情
        SysAnnouncement announcement = announcementService.getById(announcementUser.getAnnoId());
        return announcement;
    }

    private void setReaded(SysAnnouncementUser announcementUser) {
        announcementUser.setReadStatus(true);
        updateById(announcementUser);
    }

    @Override
    public void setReadedById(int id) {
        SysAnnouncementUser announcementUser = getById(id);
        setReaded(announcementUser);
    }
}
