package com.unissoft.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.unissoft.MyConstant;
import com.unissoft.constant.GrainConstant;
import com.unissoft.job.ToBeDoneJob;
import com.unissoft.mapper.SysEarlyWarningMapper;
import com.unissoft.model.SysEarlyWarning;
import com.unissoft.model.SysUserPO;
import com.unissoft.result.PageParam;
import com.unissoft.service.QuartzService;
import com.unissoft.service.SysEarlyWarningService;
import com.unissoft.utils.DateUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.json.JSONObject;
import org.quartz.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;

import static org.quartz.CalendarIntervalScheduleBuilder.calendarIntervalSchedule;
import static org.quartz.TriggerBuilder.newTrigger;

/**
 * <p>
 * 预警管理 服务实现类
 * </p>
 *
 * @author root
 * @since 2020-12-15
 */
@Service
@Slf4j
public class SysEarlyWarningServiceImpl extends ServiceImpl<SysEarlyWarningMapper, SysEarlyWarning> implements SysEarlyWarningService {

    private final String GROUP = "warnGroup";

//    private final String NOTICE_JOB_NAME_PREFIX = "noticeJob";
//
//    private final String NOTICE_TRIGGER_NAME_PREFIX = "noticeTrigger";

    private final String TO_BE_DONE_JOB_NAME_PREFIX = "toBeDoneJob";

    private final String TO_BE_DONE_TRIGGER_NAME_PREFIX = "toBeDoneTrigger";

    @Autowired
    private SysEarlyWarningMapper earlyWarningMapper;

    @Autowired
    private Scheduler scheduler;

    @Autowired
    private QuartzService quartzService;

    /**
     * 预警分页查询
     * @param pageParam 查询参数
     * @return
     */
    @Override
    public Page<SysEarlyWarning> getPage(PageParam pageParam) {
        Integer pageIndex = pageParam.getPageIndex();
        Integer pageSize = pageParam.getPageSize();
        QueryWrapper<SysEarlyWarning> ew = new QueryWrapper<>();
        if(!StringUtils.isEmpty(pageParam.getCondition())){
            JSONObject object = new JSONObject(pageParam.getCondition());
            if (object.has("warnName")) {
                String warnName = object.getString("warnName");
                if (! StringUtils.isEmpty(warnName)) {
                    ew.like("warn_name", warnName.trim());
                }
            }
        }
        ew.orderByDesc("create_time");
        Page page = new Page();
        if (pageIndex != MyConstant.ZERO && pageSize != MyConstant.ZERO) {
            page = new Page(pageIndex, pageSize);
        }
        Page<SysEarlyWarning> pageResult = earlyWarningMapper.selectPage(page, ew);
        return pageResult;
    }

    /**
     * 保存预警，如果预警状态为开启，则启动预警功能
     * @param model
     * @param userPO
     * @return
     */
    @Override
    public boolean save(SysEarlyWarning model, SysUserPO userPO) {

        validateParam(model.getInspectCycle(), "检查周期");
        validateParam(model.getWarningTime(), "预警时间");

        model.setWarnStatus(true);//新增，默认开启
//        model.setCreateBy(userPO.getuId());
//        model.setUpdateBy(userPO.getuId());
        //
        Date date = new Date();
        model.setCreateTime(date);
        model.setUpdateTime(date);
        save(model);

        //开启预警功能
        if(model.getWarnStatus()) {
            enableWarnSchedule(model);
        }

        return true;
    }


    /**
     * 更新预警，更新前先关闭预警
     * 更新后如果预警状态为开启，则启动预警功能
     * @param model
     * @param userPO
     */
    @Override
    @Transactional
    public void update(SysEarlyWarning model, SysUserPO userPO) {
        validateParam(model.getInspectCycle(), "检查周期");
        validateParam(model.getWarningTime(), "预警时间");

        //先关闭预警
        SysEarlyWarning earlyWarning = getById(model.getId());
        stopWarnSchedule(earlyWarning);

        //保留旧的预警状态、创建人、创建时间
        model.setWarnStatus(earlyWarning.getWarnStatus());
        model.setCreateBy(earlyWarning.getCreateBy());
        model.setCreateTime(earlyWarning.getCreateTime());
        //
        model.setUpdateBy(userPO.getuId());
        model.setUpdateTime(new Date());
        //
        updateById(model);

        //开启预警功能
        if(model.getWarnStatus()) {
            enableWarnSchedule(model);
        }
    }

    /**
     * 删除预警，删除前先关闭预警
     * @param id
     */
    @Override
    @Transactional
    public void deleteById(int id) {
        SysEarlyWarning earlyWarning = getById(id);
        stopWarnSchedule(earlyWarning);//关闭启动的任务调度

        removeById(id);
    }

    /**
     * 更新预警状态为true，并启动预警
     * @param id
     */
    @Override
    @Transactional
    public void enableWarn(int id) {
        SysEarlyWarning earlyWarning = getById(id);
        earlyWarning.setWarnStatus(true);
        updateById(earlyWarning);

        enableWarnSchedule(earlyWarning);
    }
    private void enableWarnSchedule(SysEarlyWarning earlyWarning) {
//        enableNoticeSchedule(earlyWarning);
        enableToBeDoneSchedule(earlyWarning);
    }

    /**
     * 更新预警状态为false，并关闭预警
     * @param id
     */
    @Override
    @Transactional
    public void stopWarn(int id) {
        SysEarlyWarning earlyWarning = getById(id);
        earlyWarning.setWarnStatus(false);
        updateById(earlyWarning);

        stopWarnSchedule(earlyWarning);
    }
    private void stopWarnSchedule(SysEarlyWarning earlyWarning) {
//        removeNoticeSchedule(earlyWarning);
        removeToBeDoneSchedule(earlyWarning);
    }

//    /**
//     * 启动通知任务调度
//     * @param earlyWarning
//     */
//    private void enableNoticeSchedule(SysEarlyWarning earlyWarning) {
//        String jobName = NOTICE_JOB_NAME_PREFIX + earlyWarning.getId();//任务名称
//        String triggerName = NOTICE_TRIGGER_NAME_PREFIX + earlyWarning.getId();//
//
//        //如果存在则删除
//        removeNoticeSchedule(earlyWarning);
//
//        String jobDesc = earlyWarning.getWarnName();                    //任务描述
//        String usersInfo = earlyWarning.getWarnTarget();                //通知对象(用户ID|用户姓名)
//        String content = earlyWarning.getWarnName() + "，请知晓！";      //通知内容
//        String timePeriod = earlyWarning.getInspectCycle();             //检查周期：每2周、每个月等
//        String aheadTime = earlyWarning.getWarningTime();               //预警时间：提前几个小时或几天
//
//        JobDetail noticeJob = JobBuilder.newJob(NoticeJob.class).withIdentity(jobName, GROUP).withDescription(jobDesc).build();
//        noticeJob.getJobDataMap().put("usersInfo", usersInfo);
//        noticeJob.getJobDataMap().put("content", content);
//
//        CalendarIntervalTrigger noticeTrigger = getNoticeCalendarIntervalTrigger(triggerName, GROUP, timePeriod, aheadTime);
//
//        try {
//            scheduler.scheduleJob(noticeJob, noticeTrigger);
//        } catch (SchedulerException e) {
//            log.error(e.getMessage());
//            throw new RuntimeException("任务调度启动失败");
//        }
//    }

//    /**
//     * 删除通知任务调度
//     * @param earlyWarning
//     */
//    private void removeNoticeSchedule(SysEarlyWarning earlyWarning) {
//        String jobName = NOTICE_JOB_NAME_PREFIX + earlyWarning.getId();//任务名称
//        String triggerName = NOTICE_TRIGGER_NAME_PREFIX + earlyWarning.getId();
//
//        removeSchedule(jobName, triggerName);
//    }

//    /**
//     * 获取通知的 CalendarIntervalTrigger
//     * @param triggerName
//     * @param group
//     * @param timePeriod
//     * @param aheadTime
//     * @return
//     */
//    private CalendarIntervalTrigger getNoticeCalendarIntervalTrigger(String triggerName, String group, String timePeriod, String aheadTime) {
//        validateParam(timePeriod, "检查周期");
//        validateParam(aheadTime, "预警时间");
//
//        String[] timePeriodParam = timePeriod.split("_");
//        String[] aheadTimeParams = aheadTime.split("_");
//        //
//        Date startAt = getStartAtDate4Notice(timePeriodParam[1], aheadTimeParams[1], Integer.parseInt(aheadTimeParams[0]));
//
//        CalendarIntervalScheduleBuilder calendarIntervalScheduleBuilder = getCalendarIntervalScheduleBuilder(timePeriodParam[1], Integer.parseInt(timePeriodParam[0]));
//        //
//        CalendarIntervalTrigger noticeCalendarTrigger = newTrigger().withIdentity(triggerName, group)
//                .startAt(startAt)//设置开始调度时间
//                .withSchedule(calendarIntervalScheduleBuilder)//设置调度频率
//                .build();
//
//        return noticeCalendarTrigger;
//    }



    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////

    /**
     * 启动待办任务调度
     * @param earlyWarning
     */
    private void enableToBeDoneSchedule(SysEarlyWarning earlyWarning) {
        String jobName = TO_BE_DONE_JOB_NAME_PREFIX + earlyWarning.getId();//任务名称
        String triggerName = TO_BE_DONE_TRIGGER_NAME_PREFIX + earlyWarning.getId();//

        //如果存在则删除
        removeToBeDoneSchedule(earlyWarning);

        String title = earlyWarning.getWarnName();                    //任务描述
        String usersInfo = earlyWarning.getWarnTarget();                //通知对象(用户ID|用户姓名)
        String timePeriod = earlyWarning.getInspectCycle();             //检查周期：每2周、每个月等
        String aheadTime = earlyWarning.getWarningTime();               //预警时间：提前几个小时或几天
        Integer levelDepotHouse = earlyWarning.getLevelDepotHouse();    //按库或仓房为单位进行预警：1库，2仓房

        JobDetail toBeDoneJob = JobBuilder.newJob(ToBeDoneJob.class).withIdentity(jobName, GROUP).withDescription(title).build();
        toBeDoneJob.getJobDataMap().put("title", title);
        toBeDoneJob.getJobDataMap().put("usersInfo", usersInfo);
        toBeDoneJob.getJobDataMap().put("timePeriod", timePeriod);//检查周期
        toBeDoneJob.getJobDataMap().put("aheadTime", aheadTime);//预警时间
        toBeDoneJob.getJobDataMap().put("levelDepotHouse", levelDepotHouse);
        toBeDoneJob.getJobDataMap().put("toPage", earlyWarning.getToPage());
        toBeDoneJob.getJobDataMap().put("type", earlyWarning.getWarnCode());//类型 用 预警Code

        CalendarIntervalTrigger toBeDoneTrigger = getToBeDoneCalendarIntervalTrigger(triggerName, GROUP, timePeriod);

        try {
            scheduler.scheduleJob(toBeDoneJob, toBeDoneTrigger);
        } catch (SchedulerException e) {
            log.error(e.getMessage());
            throw new RuntimeException("任务调度启动失败");
        }
    }

    /**
     * 删除待办任务调度
     * @param earlyWarning
     */
    private void removeToBeDoneSchedule(SysEarlyWarning earlyWarning) {
        String jobName = TO_BE_DONE_JOB_NAME_PREFIX + earlyWarning.getId();//任务名称
        String triggerName = TO_BE_DONE_TRIGGER_NAME_PREFIX + earlyWarning.getId();

        removeSchedule(jobName, triggerName);
    }

    private void removeSchedule(String jobName, String triggerName) {
        JobKey jobKey = JobKey.jobKey(jobName, GROUP);
        try {
            if(scheduler.checkExists(jobKey)) {
                quartzService.removeJob(jobName, triggerName, GROUP);
            }
        } catch (SchedulerException e) {
            e.printStackTrace();
        }
    }

    /**
     * 获取 待办任务的 CalendarIntervalTrigger
     * @param triggerName
     * @param group
     * @param timePeriod
     * @return
     */
    private CalendarIntervalTrigger getToBeDoneCalendarIntervalTrigger(String triggerName, String group, String timePeriod) {
        validateParam(timePeriod, "检查周期");

        String[] timePeriodParam = timePeriod.split("_");
        //
        Date startAt = getStartAtDate4ToBeDone(timePeriodParam[1]);

        CalendarIntervalScheduleBuilder calendarIntervalScheduleBuilder = getCalendarIntervalScheduleBuilder(timePeriodParam[1], Integer.parseInt(timePeriodParam[0]));
        //
        TriggerBuilder<CalendarIntervalTrigger> calendarIntervalTriggerBuilder = newTrigger().withIdentity(triggerName, group)
                .startAt(startAt)//设置开始调度时间
                .withSchedule(calendarIntervalScheduleBuilder);//设置调度频率

        if(timePeriodParam[1].equals(GrainConstant.STR_WEEKDAY)) {
            calendarIntervalTriggerBuilder.modifiedByCalendar("weekly");//weekly在Scheduler初始化时，已注册
        }

        CalendarIntervalTrigger toBeDoneCalendarTrigger = calendarIntervalTriggerBuilder.build();

        return toBeDoneCalendarTrigger;
    }






    /**
     * timePeriod值：2_second、2_minute、2_hour、1_day、2_week、3_month、4_season、5_year
     * @param param
     */
    private void validateParam(String param, String paramDesc) {
        if(StringUtils.isEmpty(param)) {
            throw new IllegalArgumentException(paramDesc + "值不能为空");
        }

        String[] value = param.split("_");
        if(value.length != 2) {
            throw new IllegalArgumentException(paramDesc + "值中间必须带有一个'_'");
        }

        if(! StringUtils.isNumeric(value[0])) {//是否为正整数
            throw new IllegalArgumentException(paramDesc + "值'_'前必须为正整数");
        }

        String periodType = value[1];

        if(! periodType.equals(GrainConstant.STR_SECOND) && ! periodType.equals(GrainConstant.STR_MINUTE) && ! periodType.equals(GrainConstant.STR_HOUR) &&
                ! periodType.equals(GrainConstant.STR_DAY) && ! periodType.equals(GrainConstant.STR_WEEKDAY) && ! periodType.equals(GrainConstant.STR_WEEK) &&
                ! periodType.equals(GrainConstant.STR_MONTH) && ! periodType.equals(GrainConstant.STR_SEASON) && ! periodType.equals(GrainConstant.STR_YEAR) ) {
            throw new IllegalArgumentException(paramDesc + "值'_'后的值必须为second、minute、hour、day、weekday、week、month、season、year中的一种");
        }

    }

    /**
     * 检查周期 值为：2_minute(每2分钟)、2_hour(每2小时)、1_day(每天)、2_week(每2周)、3_month(每3个月)、1_season(每季度)或者5_year(每5年)
     * 则periodType为：minute、hour、day、week、month、season或者year
     *
     * 计算待办任务的起始时间：如果检查周期为 每周或每2周，则起始时间为周一上午0点0分0秒；
     *                      如果检查周期为每个月，则起始时间为当月的1号上午0点0分0秒；
     *                      如果检查周期为每季度，则起始时间为当前季度第一个月的1号上午0点0分0秒；
     * @param periodType    检查周期类型，如 minute、hour、day、week、month、season或者year
     * @return
     */
    private static Date getStartAtDate4ToBeDone(String periodType) {
        Date startAt = null;
        if(periodType.equals(GrainConstant.STR_MINUTE)) {
            startAt = DateUtil.getCurrMinute();
        } else if(periodType.equals(GrainConstant.STR_HOUR)) {
            startAt = DateUtil.getCurrHour();
        } else if(periodType.equals(GrainConstant.STR_DAY) || periodType.equals(GrainConstant.STR_WEEKDAY)) {
            startAt = DateUtil.getCurrDate();
        } else if(periodType.equals(GrainConstant.STR_WEEK)) {
            startAt = DateUtil.getFirstDateOfCurrWeek();
        } else if(periodType.equals(GrainConstant.STR_MONTH)) {
            startAt = DateUtil.getFirstDateOfCurrMonth();
        } else if(periodType.equals(GrainConstant.STR_SEASON)) {
            startAt = DateUtil.getFirstDateOfCurrSeason();
        } else if(periodType.equals(GrainConstant.STR_YEAR)) {
            startAt = DateUtil.getFirstDateOfCurrYear();
        }

        return startAt;
    }

//    /**
//     * 检查周期 值为：2_minute(每2分钟)、2_hour(每2小时)、1_day(每天)、2_week(每2周)、3_month(每3个月)、1_season(每季度)或者5_year(每5年)
//     * 则periodType为：minute、hour、day、week、month、season或者year
//     *
//     * 预警时间 值为：2_minute(到期前2分钟)、2_hour(到期前2小时)、1_day(到期前1天)、2_week(到期前2周)、3_month(到期前3个月)、1_season(到期前1个季度)或者5_year(到期前5年)
//     * 则aheadType：minute、hour、day、week、month、season或者year
//     * aheadNum ：2、2、1、2、3、1或者5
//     *
//     * 计算通知的起始时间：如果检查周期以天、周、月、季、年为单位，则通知的时间为其对应单位第一天的8:00:00
//     *                  如检查周期是每2周，则通知时间为这2周内的第一天(星期一)8:00:00
//     *                  如检查周期是每月，则通知时间为该月第一天8:00:00
//     *
//     *                  通知的起始时间根据预警时间在其基础上再往前计算
//     *                  如到期前1天，如果检查周期是每月，则最后通知时间为该月第一天8:00:00的前一天，即上一个月最后一天的8:00:00
//     *
//     * @param periodType    periodType：minute、hour、day、week、month、season或者year
//     * @param aheadType     aheadType：minute、hour、day、week、month、season或者year
//     * @param aheadNum      aheadNum ：2、2、1、2、3、1或者5
//     * @return
//     */
//    private static Date getStartAtDate4Notice(String periodType, String aheadType, int aheadNum) {
//        Date startAt = null;
//        if(periodType.equals(GrainConstant.STR_MINUTE)) {
//            startAt = DateUtil.getCurrMinute();
//        } else if(periodType.equals(GrainConstant.STR_HOUR)) {
//            startAt = DateUtil.getCurrHour();
//        } else if(periodType.equals(GrainConstant.STR_DAY)) {
//            startAt = DateUtil.getCurrDate(8, 0, 0);
//        } else if(periodType.equals(GrainConstant.STR_WEEK)) {
//            startAt = DateUtil.getFirstDateOfCurrWeek(8, 0, 0);
//        } else if(periodType.equals(GrainConstant.STR_MONTH)) {
//            startAt = DateUtil.getFirstDateOfCurrMonth(8, 0, 0);
//        } else if(periodType.equals(GrainConstant.STR_SEASON)) {
//            startAt = DateUtil.getFirstDateOfCurrSeason(8, 0, 0);
//        } else if(periodType.equals(GrainConstant.STR_YEAR)) {
//            startAt = DateUtil.getFirstDateOfCurrYear(8, 0, 0);
//        }
//
//        //
//        if(StringUtils.isNotEmpty(aheadType) && aheadNum > 0) {
//            if(aheadType.equals(GrainConstant.STR_MINUTE)) {
//                startAt = DateUtil.aheadOfMinutes(startAt, aheadNum);
//            } else if(aheadType.equals(GrainConstant.STR_HOUR)) {
//                startAt = DateUtil.aheadOfHours(startAt, aheadNum);
//            } else if(aheadType.equals(GrainConstant.STR_DAY)) {
//                startAt = DateUtil.aheadOfDays(startAt, aheadNum);
//            } else if(aheadType.equals(GrainConstant.STR_WEEK)) {
//                startAt = DateUtil.aheadOfWeeks(startAt, aheadNum);
//            } else if(aheadType.equals(GrainConstant.STR_MONTH)) {
//                startAt = DateUtil.aheadOfMonths(startAt, aheadNum);
//            } else if(aheadType.equals(GrainConstant.STR_SEASON)) {
//                startAt = DateUtil.aheadOfSeasons(startAt, aheadNum);
//            } else if(aheadType.equals(GrainConstant.STR_YEAR)) {
//                startAt = DateUtil.aheadOfYears(startAt, aheadNum);
//            }
//        }
//        return startAt;
//    }

    private CalendarIntervalScheduleBuilder getCalendarIntervalScheduleBuilder(String periodType, int periodNum) {
        if(StringUtils.isEmpty(periodType) || periodNum < 0) {
            throw new IllegalArgumentException("periodType不能为空且periodNum必须大于0");
        }
        if(periodType.equals(GrainConstant.STR_YEAR)) {
            return calendarIntervalSchedule().withIntervalInYears(periodNum);
        } else if(periodType.equals(GrainConstant.STR_SEASON)) {
            return calendarIntervalSchedule().withIntervalInMonths(periodNum * 3);
        } else if(periodType.equals(GrainConstant.STR_MONTH)) {
            return calendarIntervalSchedule().withIntervalInMonths(periodNum);
        } else if(periodType.equals(GrainConstant.STR_WEEK)) {
            return calendarIntervalSchedule().withIntervalInWeeks(periodNum);
        } else if(periodType.equals(GrainConstant.STR_DAY) || periodType.equals(GrainConstant.STR_WEEKDAY)) {
            return calendarIntervalSchedule().withIntervalInDays(periodNum);
        } else if(periodType.equals(GrainConstant.STR_HOUR)) {
            return calendarIntervalSchedule().withIntervalInHours(periodNum);
        } else if(periodType.equals(GrainConstant.STR_MINUTE)) {
            return calendarIntervalSchedule().withIntervalInMinutes(periodNum);
        } else if(periodType.equals(GrainConstant.STR_SECOND)) {
            return calendarIntervalSchedule().withIntervalInSeconds(periodNum);
        }

        return null;
    }

}
