package com.unissoft.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.unissoft.constant.GrainConstant;
import com.unissoft.dto.MessageVO;
import com.unissoft.enums.OnlineStatusEnum;
import com.unissoft.mapper.SysMessageMapper;
import com.unissoft.model.SysMessage;
import com.unissoft.redismq.PublishService;
import com.unissoft.service.SysMessageService;
import com.unissoft.vo.UserOnlineStatusVO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Set;

/**
 * <p>
 * 消息推送表 服务实现类
 * </p>
 *
 * @author root
 * @since 2021-01-08
 */
@Service
public class SysMessageServiceImpl extends ServiceImpl<SysMessageMapper, SysMessage> implements SysMessageService {

    @Autowired
    private SysMessageMapper sysMessageMapper;

    @Autowired
    private PublishService publishService;

    @Override
    @Transactional
    public void sendMessage(MessageVO messageVO) {
        if(null != messageVO.getToUserId()) {
            SysMessage message = new SysMessage();
            BeanUtils.copyProperties(messageVO, message);
            message.setPushStatus(false);
            Date now = new Date();
            message.setCreateTime(now);
            message.setUpdateTime(now);
            //
            sysMessageMapper.insert(message);
            messageVO.setId(message.getId());
            //
            publishService.publish(GrainConstant.MQ_MSG_CHANNEL, messageVO);
        }
    }

    @Override
    @Transactional
    public void batchSendMessage(MessageVO messageVO, Set<Integer> userIds) {
        if(null != userIds && userIds.isEmpty() == false) {
            for(Integer userId : userIds) {
                messageVO.setToUserId(userId);
                sendMessage(messageVO);
            }
        }
    }

    /**
     * 获取未推送的用户消息
     * @param userId
     * @return
     */
    @Override
    public List<MessageVO> getUnpushedUserMessage(Integer userId) {
        List<MessageVO> result = null;
        QueryWrapper<SysMessage> ew = new QueryWrapper<>();
        ew.eq("to_user_id", userId);
        ew.eq("push_status", 0);
        List<SysMessage> list = sysMessageMapper.selectList(ew);

        if(null != list && list.isEmpty() == false) {
            result = new ArrayList<>();
            for(SysMessage sysMessage : list) {
                MessageVO messageVO = new MessageVO();
                BeanUtils.copyProperties(sysMessage, messageVO);
                result.add(messageVO);
            }
        }
        return result;
    }

    @Override
    public void batchSetPushedStatus(List<MessageVO> list) {
        sysMessageMapper.batchSetPushedStatus(list);
    }

    @Override
    public void setPushedStatus(Integer id) {
        sysMessageMapper.setPushedStatus(id);
    }

    @Override
    public void removeWebSocketSession(int id) {
        UserOnlineStatusVO onlineStatusVO = new UserOnlineStatusVO();
        onlineStatusVO.setUserId(id);
        onlineStatusVO.setOnlineStatus(OnlineStatusEnum.OFFLINE.getCode());
        publishService.publish(GrainConstant.MQ_USER_STATUS_CHANNEL, onlineStatusVO);
    }


}
