package com.unissoft.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.unissoft.MyConstant;
import com.unissoft.constant.GrainConstant;
import com.unissoft.mapper.SysNoticeMapper;
import com.unissoft.model.SysNotice;
import com.unissoft.result.PageParam;
import com.unissoft.service.SysNoticeService;
import com.unissoft.service.SysNoticeUserService;
import com.unissoft.utils.DateUtil;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.time.DateUtils;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author root
 * @since 2020-12-07
 */
@Service
public class SysNoticeServiceImpl extends ServiceImpl<SysNoticeMapper, SysNotice> implements SysNoticeService {

    @Autowired
    private SysNoticeMapper noticeMapper;

    @Autowired
    private SysNoticeUserService noticeUserService;

    @Override
    public Page<SysNotice> getPage(PageParam pageParam) {
        Integer pageIndex = pageParam.getPageIndex();
        Integer pageSize = pageParam.getPageSize();
        QueryWrapper<SysNotice> ew = new QueryWrapper<>();
        if(!StringUtils.isEmpty(pageParam.getCondition())){
            JSONObject object = new JSONObject(pageParam.getCondition());

            if (object.has("noticeTimeStart")) {
                String noticeTimeStartStr = object.getString("noticeTimeStart");
                if (! StringUtils.isEmpty(noticeTimeStartStr)) {
                    Date noticeTimeStart = DateUtil.formatDate(noticeTimeStartStr, "yyyy-MM-dd");
                    ew.ge("notice_time", noticeTimeStart);
                }
            }

            if (object.has("noticeTimeEnd")) {
                String noticeTimeEndStr = object.getString("noticeTimeEnd");
                if (! StringUtils.isEmpty(noticeTimeEndStr)) {
                    Date noticeTimeEnd = DateUtil.formatDate(noticeTimeEndStr, "yyyy-MM-dd");
                    noticeTimeEnd = DateUtils.addDays(noticeTimeEnd, 1);//加1天
                    ew.lt("notice_time", noticeTimeEnd);
                }
            }

            if (object.has("noticeType")) {
                String noticeType = object.getString("noticeType");
                if (! StringUtils.isEmpty(noticeType)) {
                    ew.eq("notice_type", noticeType.trim());
                }
            }

            if (object.has(GrainConstant.STR_CONTENT)) {
                String content = object.getString(GrainConstant.STR_CONTENT);
                if (! StringUtils.isEmpty(content)) {
                    ew.like(GrainConstant.STR_CONTENT, content.trim());
                }
            }
        }
        ew.orderByDesc("create_time");
        Page page = new Page();
        if (pageIndex != MyConstant.ZERO && pageSize != MyConstant.ZERO) {
            page = new Page(pageIndex, pageSize);
        }
        Page<SysNotice> pageResult = noticeMapper.selectPage(page, ew);
        return pageResult;
    }

    @Override
    @Transactional
    public void removeNoticeById(int id) {
        SysNotice notice = getById(id);
        removeById(id);

        noticeUserService.removeByNoticeId(notice.getId());
    }

}
