package com.unissoft.service.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Maps;
import com.unissoft.MyConstant;
import com.unissoft.constant.GrainConstant;
import com.unissoft.dto.MessageVO;
import com.unissoft.dto.SendNoticeVO;
import com.unissoft.dto.SysNoticeVO;
import com.unissoft.enums.MessageTypeEnum;
import com.unissoft.enums.NoticeTypeEnum;
import com.unissoft.mapper.SysNoticeUserMapper;
import com.unissoft.model.SysNotice;
import com.unissoft.model.SysNoticeUser;
import com.unissoft.result.PageParam;
import com.unissoft.service.SysMessageService;
import com.unissoft.service.SysNoticeService;
import com.unissoft.service.SysNoticeUserService;
import com.unissoft.vo.SysNoticeUserVO;
import org.apache.commons.lang.StringUtils;
import org.json.JSONObject;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author root
 * @since 2020-12-07
 */
@Service
public class SysNoticeUserServiceImpl extends ServiceImpl<SysNoticeUserMapper, SysNoticeUser> implements SysNoticeUserService {

    @Autowired
    private SysNoticeUserMapper noticeUserMapper;

    @Autowired
    private SysNoticeService noticeService;

    @Autowired
    private SysMessageService sysMessageService;

    @Override
    public Page<SysNoticeUserVO> getPage(PageParam pageParam, Integer userId) {
        Integer pageIndex = pageParam.getPageIndex();
        Integer pageSize = pageParam.getPageSize();
        Map<String, Object> map = Maps.newHashMap();
        if (null != userId) {
            map.put("userId", userId);
        }

        if (! StringUtils.isEmpty(pageParam.getCondition())) {
            JSONObject object = new JSONObject(pageParam.getCondition());

            if (object.has("readStatus")) {
                Boolean readStatus = object.getBoolean("readStatus");
                if (null != readStatus) {
                    map.put("readStatus", readStatus ? 1 : 0 );
                }
            }

            if (object.has("noticeType")) {
                String noticeType = object.getString("noticeType");
                if (! StringUtils.isEmpty(noticeType)) {
                    map.put("noticeType", noticeType.trim());
                }
            }

            if (object.has(GrainConstant.STR_CONTENT)) {
                String content = object.getString(GrainConstant.STR_CONTENT);
                if (! StringUtils.isEmpty(content)) {
                    map.put(GrainConstant.STR_CONTENT, "%"+content.trim()+"%");
                }
            }
        }

        Page page = new Page();
        if (pageIndex != MyConstant.ZERO && pageSize != MyConstant.ZERO) {
            page = new Page(pageIndex, pageSize);
        }
        Page<SysNoticeUserVO> pageResult = noticeUserMapper.getPage(page, map);

        return pageResult;
    }

    @Override
    public boolean saveNoticeUser(SysNoticeUser model) {
        model.setReadStatus(false);//设置未读
        Date date = new Date();
        model.setCreateTime(date);
        model.setUpdateTime(date);
        save(model);
        return true;
    }

    @Override
    public boolean updateNoticeUser(SysNoticeUser model) {
        Date date = new Date();
        model.setUpdateTime(date);
        updateById(model);
        return true;
    }

    @Override
    @Transactional
    public void sendNotice(SendNoticeVO sendNoticeVO) {
        Date date = new Date();
        SysNoticeVO sysNoticeVO = sendNoticeVO.getSysNoticeVO();
        if(null != sysNoticeVO) {
            SysNotice sysNotice = new SysNotice();
            BeanUtils.copyProperties(sysNoticeVO, sysNotice);
            sysNotice.setCreateTime(date);
            sysNotice.setUpdateTime(date);
            noticeService.save(sysNotice);

            Map<Integer, String> users = sendNoticeVO.getUsers();
            if(null != users && users.isEmpty() == false) {
                SysNoticeUser sysNoticeUser = new SysNoticeUser();
                sysNoticeUser.setNoticeId(sysNotice.getId());
                sysNoticeUser.setCreateTime(date);
                sysNoticeUser.setUpdateTime(date);
                sysNoticeUser.setReadStatus(false);
                for(Integer userId : users.keySet()) {
                    sysNoticeUser.setUserId(userId);
                    sysNoticeUser.setUserName(users.get(userId));
                    //
                    save(sysNoticeUser);
                }
                //
                MessageVO messageVO = getMessageVO(sysNotice);
                sysMessageService.batchSendMessage(messageVO, users.keySet());
            }
        }

    }

    private MessageVO getMessageVO(SysNotice sysNotice) {

        MessageVO messageVO = new MessageVO();
        messageVO.setTitle(sysNotice.getTitle());
        messageVO.setContent(sysNotice.getContent());

        Integer noticeType = sysNotice.getNoticeType();
        if(null != noticeType) {
            if(noticeType == NoticeTypeEnum.SYS.getCode() || noticeType == NoticeTypeEnum.TODO.getCode()) {
                messageVO.setType(MessageTypeEnum.ATTENTION.getCode());
            } else if(noticeType == NoticeTypeEnum.APPROVED.getCode()) {
                messageVO.setType(MessageTypeEnum.FINISH.getCode());
            } else if(noticeType == NoticeTypeEnum.DISAGREE.getCode() || noticeType == NoticeTypeEnum.WARN.getCode()) {
                messageVO.setType(MessageTypeEnum.WARN.getCode());
            }
        }

        return messageVO;
    }

    @Override
    public void removeByNoticeId(Integer noticeId) {
        Map<String, Object> columnMap = new HashMap<>();
        columnMap.put("notice_id", noticeId);
        removeByMap(columnMap);
    }

    private void setReaded(SysNoticeUser noticeUser) {
        noticeUser.setReadStatus(true);
        updateById(noticeUser);
    }

    @Override
    public void setReadedById(int id) {
        SysNoticeUser noticeUser = getById(id);
        setReaded(noticeUser);
    }

    @Override
    public SysNotice viewNoticeById(int id) {
        SysNoticeUser noticeUser = getById(id);
        //设置已读
        setReaded(noticeUser);
        //获取通知详情
        SysNotice notice = noticeService.getById(noticeUser.getNoticeId());
        return notice;
    }


}
