package com.unissoft.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.unissoft.MyConstant;
import com.unissoft.dto.SendNoticeVO;
import com.unissoft.dto.SysNoticeVO;
import com.unissoft.feign.SystemManageFeign;
import com.unissoft.mapper.SysPolicySystemMapper;
import com.unissoft.model.SysFileMetadata;
import com.unissoft.model.SysPolicySystem;
import com.unissoft.model.SysPolicySystemUser;
import com.unissoft.model.SysUserPO;
import com.unissoft.result.PageParam;
import com.unissoft.service.SysFileMetadataService;
import com.unissoft.service.SysNoticeUserService;
import com.unissoft.service.SysPolicySystemService;
import com.unissoft.service.SysPolicySystemUserService;
import com.unissoft.utils.DateUtil;
import com.unissoft.vo.SysPolicySystemVO;
import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.time.DateUtils;
import org.json.JSONObject;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 政策制度 服务实现类
 * </p>
 *
 * @author root
 * @since 2021-01-26
 */
@Service
public class SysPolicySystemServiceImpl extends ServiceImpl<SysPolicySystemMapper, SysPolicySystem> implements SysPolicySystemService {

    @Autowired
    private SysPolicySystemMapper policySystemMapper;

    @Autowired
    private SysPolicySystemUserService policySystemUserService;

    @Autowired
    private SystemManageFeign systemManageFeign;

    @Autowired
    private SysNoticeUserService noticeUserService;

    @Autowired
    private SysFileMetadataService fileMetadataService;


    @Override
    public Page<SysPolicySystem> getPage(PageParam pageParam, SysUserPO userPO) {
        Integer pageIndex = pageParam.getPageIndex();
        Integer pageSize = pageParam.getPageSize();
        QueryWrapper<SysPolicySystem> ew = new QueryWrapper<>();

        ew.ge("publish_org_id", userPO.getDepotId());
        if(! StringUtils.isEmpty(pageParam.getCondition())) {
            JSONObject object = new JSONObject(pageParam.getCondition());

            if (object.has("publishTimeStart")) {
                String publishTimeStartStr = object.getString("publishTimeStart");
                if (! StringUtils.isEmpty(publishTimeStartStr)) {
                    Date publishTimeStart = DateUtil.formatDate(publishTimeStartStr, "yyyy-MM-dd");
                    ew.ge("publish_time", publishTimeStart);
                }
            }

            if (object.has("publishTimeEnd")) {
                String publishTimeEndStr = object.getString("publishTimeEnd");
                if (! StringUtils.isEmpty(publishTimeEndStr)) {
                    Date publishTimeEnd = DateUtil.formatDate(publishTimeEndStr, "yyyy-MM-dd");
                    publishTimeEnd = DateUtils.addDays(publishTimeEnd, 1);//加1天
                    ew.lt("publish_time", publishTimeEnd);
                }
            }

            if (object.has("title")) {
                String title = object.getString("title");
                if (! StringUtils.isEmpty(title)) {
                    ew.like("title", title.trim());
                }
            }
        }
        ew.orderByDesc("publish_time");
        Page page = new Page();
        if (pageIndex != MyConstant.ZERO && pageSize != MyConstant.ZERO) {
            page = new Page(pageIndex, pageSize);
        }
        Page<SysPolicySystem> pageResult = policySystemMapper.selectPage(page, ew);
        return pageResult;

    }

    @Override
    public SysPolicySystemVO getDataById(int id) {
        SysPolicySystemVO result = null;
        SysPolicySystem policySystem = getById(id);
        if(null != policySystem) {
            result = new SysPolicySystemVO();
            BeanUtils.copyProperties(policySystem, result);
            if(StringUtils.isNotEmpty(result.getAttachments())) {
                List<SysFileMetadata> list = new ArrayList<>();
                String[] attachmentArr = result.getAttachments().split(",");
                for(String attachment : attachmentArr) {
                    SysFileMetadata fileMetadata = fileMetadataService.getById(attachment);
                    if(null != fileMetadata) list.add(fileMetadata);
                }
                result.setFileMetadataList(list);
            }
        }
        return result;
    }

    @Override
    @Transactional
    public boolean add(SysPolicySystem model, SysUserPO userPO) {
        model.setPublishUserId(userPO.getuId());
        model.setPublishUserName(userPO.getPersonnelName());
        model.setPublishOrgId(userPO.getDepotId());
        model.setPublishOrgName(userPO.getDepotName());

        model.setCreateBy(userPO.getuId());
        model.setUpdateBy(userPO.getuId());

        Date date = new Date();
        model.setCreateTime(date);
        model.setUpdateTime(date);
        model.setPublishTime(date);//发布时间为新增时的系统时间
        //
        save(model);

        Map<Integer, String> users = getNoticeUserInfo(userPO.getDepotId(), "10", null);//接收对象：recvType=10 全员
        saveSysPolicySystemUsers(model.getId(), users);

        //如果需要通知
        if(model.getIsNotice()) {
            sendMessage4PolicySystemUsers(model, userPO, users);
        }

        return true;
    }

    @Override
    @Transactional
    public boolean updateById(SysPolicySystem model, SysUserPO userPO) {
        model.setPublishUserId(userPO.getuId());
        model.setPublishUserName(userPO.getPersonnelName());
        model.setPublishOrgId(userPO.getDepotId());
        model.setPublishOrgName(userPO.getDepotName());

        model.setUpdateBy(userPO.getuId());

        Date date = new Date();
        model.setUpdateTime(date);
        model.setPublishTime(date);//发布时间为新增时的系统时间
        //
        updateById(model);

        Map<Integer, String> users = getNoticeUserInfo(userPO.getDepotId(), "10", null);//接收对象：recvType=10 全员
        //如果需要通知
        if(model.getIsNotice()) {
            sendMessage4PolicySystemUsers(model, userPO, users);
        }

        return true;
    }

    private void saveSysPolicySystemUsers(int policyId, Map<Integer, String> users) {
        if(null != users && users.isEmpty() == false) {
            List<SysPolicySystemUser> list = new ArrayList<>();
            for(Integer userId : users.keySet()) {
                SysPolicySystemUser policySystemUser = new SysPolicySystemUser();
                policySystemUser.setPolicyId(policyId);
                policySystemUser.setUserId(userId);
                policySystemUser.setUserName(users.get(userId));

                Date date = new Date();
                policySystemUser.setCreateTime(date);
                policySystemUser.setUpdateTime(date);

                list.add(policySystemUser);
            }
            policySystemUserService.batchSave(list);
        }
    }

    private Map<Integer, String> getNoticeUserInfo(Integer depotId, String recvType, String recvDetails) {
        Map<Integer, String> users = systemManageFeign.getNoticeUserInfo(depotId, recvType, recvDetails);
        return users;
    }


    /**
     *
     * @param model
     * @param userPO
     */
    private void sendMessage4PolicySystemUsers(SysPolicySystem model, SysUserPO userPO, Map<Integer, String> users) {
        if(null != users && users.isEmpty() == false) {
            SendNoticeVO sendNoticeVO = new SendNoticeVO();
            sendNoticeVO.setUsers(users);

            SysNoticeVO noticeVO = new SysNoticeVO();
            noticeVO.setNoticeType(50);//通知类型：10系统、20待办、30审批通过、40审批不通过、50警告
            noticeVO.setNoticeTime(new Date());
            noticeVO.setTitle(model.getTitle());
            noticeVO.setContent(model.getTitle());
            noticeVO.setOrgName(userPO.getDepotName());

            sendNoticeVO.setSysNoticeVO(noticeVO);
            noticeUserService.sendNotice(sendNoticeVO);
        }
    }

    @Override
    @Transactional
    public void removePolicySystemById(int id) {
        removeById(id);
        //删除政策制度用户表
        policySystemUserService.removeByPolicySystemId(id);
    }

}
