package com.unissoft.service.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Maps;
import com.unissoft.MyConstant;
import com.unissoft.constant.GrainConstant;
import com.unissoft.mapper.SysPolicySystemUserMapper;
import com.unissoft.model.SysPolicySystemUser;
import com.unissoft.model.SysUserPO;
import com.unissoft.result.PageParam;
import com.unissoft.service.SysPolicySystemService;
import com.unissoft.service.SysPolicySystemUserService;
import com.unissoft.vo.SysPolicySystemUserVO;
import com.unissoft.vo.SysPolicySystemVO;
import org.apache.commons.lang.StringUtils;
import org.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 政策制度用户表 服务实现类
 * </p>
 *
 * @author root
 * @since 2021-01-26
 */
@Service
public class SysPolicySystemUserServiceImpl extends ServiceImpl<SysPolicySystemUserMapper, SysPolicySystemUser> implements SysPolicySystemUserService {

    @Autowired
    private SysPolicySystemUserMapper policySystemUserMapper;

    @Autowired
    private SysPolicySystemService policySystemService;

    @Override
    public Page<SysPolicySystemUserVO> getPage(PageParam pageParam, SysUserPO userPO) {
        Integer pageIndex = pageParam.getPageIndex();
        Integer pageSize = pageParam.getPageSize();
        Map<String, Object> map = Maps.newHashMap();
        map.put("userId", userPO.getuId());

        if (! StringUtils.isEmpty(pageParam.getCondition())) {
            JSONObject object = new JSONObject(pageParam.getCondition());

            if (object.has("title")) {
                String title = object.getString("title");
                if (! StringUtils.isEmpty(title)) {
                    map.put("title", "%"+title.trim()+"%");
                }
            }

            if (object.has(GrainConstant.STR_CONTENT)) {
                String content = object.getString(GrainConstant.STR_CONTENT);
                if (! StringUtils.isEmpty(content)) {
                    map.put(GrainConstant.STR_CONTENT, "%"+content.trim()+"%");
                }
            }
        }

        Page page = new Page();
        if (pageIndex != MyConstant.ZERO && pageSize != MyConstant.ZERO) {
            page = new Page(pageIndex, pageSize);
        }
        Page<SysPolicySystemUserVO> pageResult = policySystemUserMapper.getPage(page, map);

        return pageResult;
    }

    @Override
    @Transactional
    public int batchSave(List<SysPolicySystemUser> list) {
        return baseMapper.insertBatch(list);
    }

    @Override
    public void removeByPolicySystemId(int annoId) {
        Map<String, Object> columnMap = new HashMap<>();
        columnMap.put("policy_id", annoId);
        removeByMap(columnMap);
    }

    @Override
    public SysPolicySystemVO viewPolicySystemById(int id) {
        SysPolicySystemUser policySystemUser = getById(id);
        //获取政策制度详情
        SysPolicySystemVO policySystemVO = policySystemService.getDataById(policySystemUser.getPolicyId());
        return policySystemVO;
    }

}
